<?php

namespace Drupal\cache_table_rebuild\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Controller for rebuilding cache tables.
 */
class CacheTableRebuildController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = new static();
    $instance->connection = $container->get('database');
    return $instance;
  }

  /**
   * Builds the cache table rebuild page.
   */
  public function build() {
    $connection = $this->connection;

    // Get the database name.
    $database_name = $connection->getConnectionOptions()['database'];

    // Get the table prefix.
    $options = $connection->getConnectionOptions();
    $prefix = '';
    if (!empty($options['prefix'])) {
      if (is_array($options['prefix'])) {
        $prefix = $options['prefix']['default'] ?? '';
      }
      else {
        $prefix = $options['prefix'];
      }
    }

    // Find all tables matching the pattern, including the prefix.
    $pattern = $prefix . 'cache%';
    $query = $connection->query("
      SELECT TABLE_NAME
      FROM information_schema.TABLES
      WHERE TABLE_SCHEMA = :schema AND TABLE_NAME LIKE :pattern
    ", [
      ':schema' => $database_name,
      ':pattern' => $pattern,
    ]);

    $tables = $query->fetchAll(\PDO::FETCH_COLUMN);

    $cache_tables = [];
    foreach ($tables as $table_name) {
      // Remove the prefix from the table name for display purposes.
      $display_table_name = substr($table_name, strlen($prefix));

      // Get the size of each table.
      $size = $connection->query("
        SELECT ROUND(((DATA_LENGTH + INDEX_LENGTH) / 1024 / 1024), 2) AS size_mb
        FROM information_schema.TABLES
        WHERE TABLE_SCHEMA = :schema AND TABLE_NAME = :table
      ", [
        ':schema' => $database_name,
        ':table' => $table_name,
      ])->fetchField();

      // Convert size to a float value for sorting.
      $size = (float) $size;

      $rebuild_url = Url::fromRoute('cache_table_rebuild.rebuild_confirm', ['table_name' => $display_table_name]);
      $rebuild_link = Link::fromTextAndUrl($this->t('Rebuild'), $rebuild_url)->toString();

      $cache_tables[] = [
        'name' => $display_table_name,
        'size_mb' => $size,
        'operations' => $rebuild_link,
      ];
    }

    // Sort the cache tables by size_mb in descending order.
    usort($cache_tables, function ($a, $b) {
      return $b['size_mb'] <=> $a['size_mb'];
    });

    // Build the table header.
    $header = [
      $this->t('Table Name'),
      $this->t('Size (MB)'),
      $this->t('Operations'),
    ];
    $rows = [];

    foreach ($cache_tables as $cache_table) {
      $rows[] = [
        'data' => [
          $cache_table['name'],
          number_format($cache_table['size_mb'], 2),
          $cache_table['operations'],
        ],
      ];
    }

    // Build the render array.
    $build['cache_tables'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No cache tables found.'),
    ];

    return $build;
  }

}
