<?php

namespace Drupal\cache_table_rebuild\Form;

use Drupal\Core\Form\ConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Database\Connection;

/**
 * Form to confirm cache table rebuild operations.
 */
class CacheTableRebuildConfirmForm extends ConfirmFormBase {

  /**
   * The name of the table to rebuild (without prefix).
   *
   * @var string
   */
  protected $tableName;

  /**
   * The full table name including prefix.
   *
   * @var string
   */
  protected $fullTableName;

  /**
   * The table prefix.
   *
   * @var string
   */
  protected $prefix;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Constructs a new CacheTableRebuildConfirmForm.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   */
  public function __construct(Connection $connection) {
    $this->connection = $connection;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    // Instantiate this form class.
    return new static(
      $container->get('database')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cache_table_rebuild_confirm_form';
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to rebuild the %table table?', ['%table' => $this->tableName]);
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url('cache_table_rebuild.list');
  }

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Rebuild');
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $table_name = NULL) {
    if (strpos($table_name, 'cache') !== 0) {
      throw new AccessDeniedHttpException();
    }
    $this->tableName = $table_name;

    // Use the injected connection.
    $connection = $this->connection;
    $options = $connection->getConnectionOptions();
    $this->prefix = '';
    if (!empty($options['prefix'])) {
      if (is_array($options['prefix'])) {
        $this->prefix = $options['prefix']['default'] ?? '';
      }
      else {
        $this->prefix = $options['prefix'];
      }
    }
    $this->fullTableName = $this->prefix . $this->tableName;

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Use the injected connection.
    $connection = $this->connection;

    // Escape table names to prevent SQL injection.
    $table_name = $connection->escapeTable($this->fullTableName);
    $new_table = $connection->escapeTable($this->prefix . 'new_' . $this->tableName);
    $old_table = $connection->escapeTable($this->prefix . 'old_' . $this->tableName);

    // Store the current innodb_lock_wait_timeout value.
    $original_timeout = (int) $connection->query("SHOW VARIABLES LIKE 'innodb_lock_wait_timeout'")->fetchField(1);

    // Set a low innodb_lock_wait_timeout for this session.
    $low_timeout = 1; // Timeout in seconds.
    $connection->query("SET innodb_lock_wait_timeout = $low_timeout");

    try {
      // Drop temporary tables if they exist.
      $connection->query("DROP TABLE IF EXISTS $new_table");
      $connection->query("DROP TABLE IF EXISTS $old_table");

      // Proceed with creating the new table.
      $connection->query("CREATE TABLE $new_table LIKE $table_name");

      // Attempt to rename tables.
      $connection->query("RENAME TABLE $table_name TO $old_table, $new_table TO $table_name");

      // Drop the old table.
      $connection->query("DROP TABLE $old_table");

      $this->messenger()->addStatus($this->t('The %table table has been rebuilt.', ['%table' => $this->tableName]));
    }
    catch (\Exception $e) {
      // Check if the exception is due to a lock wait timeout.
      if (strpos($e->getMessage(), 'Lock wait timeout exceeded') !== FALSE) {
        $this->messenger()->addError($this->t('The %table table is currently in use and cannot be rebuilt at this time. Please try again later.', ['%table' => $this->tableName]));
      }
      else {
        $this->messenger()->addError($this->t('An error occurred while rebuilding %table: %message', [
          '%table' => $this->tableName,
          '%message' => $e->getMessage(),
        ]));
      }
    }
    finally {
      // Restore the original innodb_lock_wait_timeout value.
      $connection->query("SET innodb_lock_wait_timeout = $original_timeout");
    }

    $form_state->setRedirect('cache_table_rebuild.list');
  }

}
